// File: IBMPToBDF.h
//
// Header File for the IBMPToBDF interface
//
// Copyright (c) 2008
//
#ifndef _IBMPTOBDF_HEADER_
#define _IBMPTOBDF_HEADER_

#ifdef _WINDOWS
#include <windows.h>
#endif // _WINDOWS
#include "IFaceDefs.h"

#define  JIID_IBMPTOBDF_V1       0x83726780
#define  JCLSID_IBMPTOBDF_V1     0x83726781

#define  BMPTOBDFERR_WRONGFILETYPE           -1
#define  BMPTOBDFERR_FILEACCESS              -2
#define  BMPTOBDFERR_FILENAMELENGTH          -3
#define  BMPTOBDFERR_FILENOTSUPPORTED        -4
#define  BMPTOBDFERR_BADFILEDATA             -5
#define  BMPTOBDFERR_OUTOFMEMORY             -6
#define  BMPTOBDFERR_FONTBITSFAILED          -7


typedef struct BMPBDFFileInfo BMPBDFFileInfo;

struct BMPBDFFileInfo{
   char * szBMPFile;
   char * szBDFFile;
};

typedef struct BMPFontOptions BMPFontOptions;

struct BMPFontOptions{
   char *               szFontName;
   unsigned short       wFirstGlyphValue;
   unsigned char        cbGlyphAllowedVerticalBlanks;
   unsigned char        cbDescentPixels;
   unsigned char        cbGenerateFixedWidth;
   unsigned char        cbUpperBlankPad;
   unsigned char        cbLowerBlankPad;
   unsigned char        cbLeftBlankPad;
   unsigned char        cbRightBlankPad;
};

typedef struct IBMPToBDF IBMPToBDF;

JIFACE(IBMPToBDF)
{
   JQI_TYPE(IBMPToBDF);

   unsigned int   (*ConvertBMPFileToBDF)(IBMPToBDF * po, const BMPBDFFileInfo * pfiInfo, const BMPFontOptions * pbfOpt, unsigned int * pnDetailedError);
};

#define IBMPTOBDF_AddRef(p)                            GETVT((p),IBMPToBDF)->AddRef((p))
#define IBMPTOBDF_Release(p)                           GETVT((p),IBMPToBDF)->Release((p))
#define IBMPTOBDF_QueryInterface(p,c,o)                GETVT((p),IBMPToBDF)->QueryInterface((p),(c),(o))
#define IBMPTOBDF_ConvertBMPFileToBDF(p,i,o,e)         GETVT((p),IBMPToBDF)->ConvertBMPFileToBDF((p),(i),(o),(e))


// ***************************************************************************************************************************
// IBMPToBDF.h Interface File Header for BMP to BDF Converting
// ***************************************************************************************************************************
//
// ***************************************************************************************************************************
// IBMPToBDF Interface
// ***************************************************************************************************************************
// 
// This interface converts a single line strip of bitmap characters into a usable BDF font.
// BDF stands for Bitmap Distribution Format which are non-scalar fonts usually used in embedded 
// devices for redering fonts. Additionally, X Windows on Unix can use BDF fonts.
//
// Usage:
// You supply the file name and path to a bitmap with the glyphs as an input to this converter.
// Additionally, some parameters have to come in to descrive the values to assign to each respective
// glyph. The main working function is: IBMPTOBDF_ConvertBMPFileToBDF. Please see the doc for that
// function for more detailed information. Additionally, see the extra documentation for the type
// of input bitmap expected.
//
// ---------------------------------------------------------------------------------------------------------------------------
// IBMPTOBDF_AddRef
// ---------------------------------------------------------------------------------------------------------------------------
// 
// unsigned int IBMPTOBDF_AddRef(IBMPToBDF * po);
//
// Description:
// Adds a reference to the interface. Useful for when sharing the interface pointer amongst 
// functions and interfaces. Let's each owner manage their own instance handling of the object.
// 
// Parameters:
// po          -  Interface pointer to add reference to
//
// Return Values:
// unsigned int      -  Current reference count
// 
// ---------------------------------------------------------------------------------------------------------------------------
// IBMPTOBDF_Release
// ---------------------------------------------------------------------------------------------------------------------------
// 
// unsigned int IBMPTOBDF_Release(IBMPToBDF * po);
//
// Description:
// Removes a reference to the interface. Useful for when sharing the interface pointer amongst 
// functions and interfaces. Let's each owner manage their own instance handling of the object.
// 
// Parameters:
// po          -  Interface pointer to remove reference from
//
// Return Values:
// unsigned int      -  Current reference count, when 0 object is no longer valid
// 
// ---------------------------------------------------------------------------------------------------------------------------
// IBMPTOBDF_QueryInterface
// ---------------------------------------------------------------------------------------------------------------------------
// 
// unsigned int IBMPTOBDF_QueryInterface(IBMPToBDF * po, unsigned int nClassId, void ** ppObj);
//
// Description:
// Obtains an interface from this object given by the Class Id. This Class Id can be an interface
// pointer or a class pointer. Common usage is to get a specific version of an interface from the
// object. Additionally, one may get an object passed to them and want a specific interface style
// from that object rather than a specific version. The object requested will have its reference
// count increased on successful completion of the function. Release the returned interface when 
// finished with it to avoid leakage of memory.
// 
// Parameters:
// po          -  Interface pointer to query interfaces from
// nClassId    -  Interface or Class Id of the service requested from this interface
// ppObj       -  Out parameter to store the resulting interface on successful completion
//
// Return Values:
// NA_SUCCESS        -  Successful completion. ppObj is an interface pointer of the 
//                      service requested.
// NAERR_NOTSUPPORT  -  Supplied Interface or Class Id not supported by this interface.
// NAERR_BADPARM     -  Parameter bad. Probably ppObj was NULL.
//
// ---------------------------------------------------------------------------------------------------------------------------
// IBMPTOBDF_ConvertBMPFileToBDF
// ---------------------------------------------------------------------------------------------------------------------------
// 
// unsigned int IBMPTOBDF_ConvertBMPFileToBDF(IBMPToBDF * po,
//                                            const BMPBDFFileInfo * pfiInfo,
//                                            const BMPFontOptions * pbfOpt,
//                                            unsigned int * pnDetailedError);
//
// Description:
// Converts a bitmap file to a BDF style font based on parameters and options given. Returns an
// extended error code that can be used to inform the user of the error in translation that has
// occurred to help them fix the issue on the next attempt. This function will only recognize
// black and non-black pixels. Black pixels indicate a drawn pixel. All non-black pixels will be 
// treated the same as a non-drawn pixel.
// 
// Parameters:
// po                -  Interface pointer to query interfaces from
// pfiInfo           -  Required Input BMP filename and path and optional BDF File and path.
//                      If the BDF file parameter is NULL, then the BMP file name will be used
//                      in generating the file name. It will replace the extension with .bdf if
//                      one exists. Otherwise, it will add .bdf to the end.
// pbfOpt            -  Options for conveting the BMP to the BDF. See the description of the
//                      BMPFontOptions structure for each parameter and requirements for use.
// pnDetailedError   -  Optional detailed error information based on the failure point of
//                      the conversion. It will only be filled when supplied and when an error
//                      is returned. See BMPTOBDFERR docs for the types and descriptions.
//
// Return Values:
// NA_SUCCESS        -  Successful completion. ppObj is an interface pointer of the 
//                      service requested.
// NAERR_BADPARM     -  Parameter bad. Probably ppObj was NULL. Will not use pnDetailed Error.
// NAERR_MEMORY      -  Insufficient memory to process the conversion. If supplied, 
//                      check pnDetailed for details.
// NAERR_FAILED      -  General failure error. If supplied, check pnDetailed for details.
// NAERR_FILE        -  General file related error. If supplied, check pnDetailed for details.
//
// ***************************************************************************************************************************
// IBMPToBDF Definitions
// ***************************************************************************************************************************
//
// ---------------------------------------------------------------------------------------------------------------------------
// Interface and Class Definitions:
// ---------------------------------------------------------------------------------------------------------------------------
//
// JIID_IBMPTOBDF_V1    -  Interface Id for version 1.1of this interface. Can't instantiate.
// JCLSID_IBMPTOBDF_V1  -  Class Id for this implementation of version 1.0 of this interface
//
// ---------------------------------------------------------------------------------------------------------------------------
// Conversion Error Definitions:
// ---------------------------------------------------------------------------------------------------------------------------
//
// BMPTOBDFERR_WRONGFILETYPE     -  Bitmap file type is not correct. Contents may not be accurate.
// BMPTOBDFERR_FILEACCESS        -  Can't access BMP File for reading or BDF file for writing.
// BMPTOBDFERR_FILENAMELENGTH    -  File name is too long for this system.
// BMPTOBDFERR_FILENOTSUPPORTED  -  File contents are not supported by this implementation.
// BMPTOBDFERR_BADFILEDATA       -  File data is corrupt or otherwise unusable.
// BMPTOBDFERR_OUTOFMEMORY       -  Not enough memory available to perform operation.
// BMPTOBDFERR_FONTBITSFAILED    -  Failure in processing the font bits.
// 
// ***************************************************************************************************************************
// IBMPToBDF Structures
// ***************************************************************************************************************************
//
// ---------------------------------------------------------------------------------------------------------------------------
// BMPBDFFileInfo
// ---------------------------------------------------------------------------------------------------------------------------
// 
// struct BMPBDFFileInfo{
//    char * szBMPFile;
//    char * szBDFFile;
// };
//
// Description:
// File mapping for an input BMP File and output BDF File.
// 
// Structure Members:
// szBMPFile   -  Used with:
//                ConvertBMPFileToBDF  -  Required input parameter, file and path to input file
// szBDFFile   -  Used with:
//                ConvertBMPFileToBDF  -  Optional input parameter, file and path to output file
//                                        If not supplied or empty, name will be based on szBMPFile
// 
// ---------------------------------------------------------------------------------------------------------------------------
// BMPFontOptions
// ---------------------------------------------------------------------------------------------------------------------------
// 
// struct BMPFontOptions{
//    char *               szFontName;
//    unsigned short       wFirstGlyphValue;
//    unsigned char        cbGlyphAllowedVerticalBlanks;
//    unsigned char        cbDescentPixels;
//    unsigned char        cbGenerateFixedWidth;
//    unsigned char        cbUpperBlankPad;
//    unsigned char        cbLowerBlankPad;
//    unsigned char        cbLeftBlankPad;
//    unsigned char        cbRightBlankPad;
// };
//
// Description:
// Options to use when creating the BDF file from the boitmap pixels. It describes some of the
// parameters for the BDF that could not be contained within the simple bitmap.
// Some terms may not be familiar to you if you didn't work with fonts before, so please see
// the extra documentation section for further description of the terms.
//
// Structure Members:
// szFontName        -  Used with:
//                      ConvertBMPFileToBDF  -  Optional input parameter. Name of file inside BDF
//                                              FONT attribute. If not provided, default will be
//                                              No-Name-Font.
// wFirstGlyphValue  -  Used with:
//                      ConvertBMPFileToBDF  -  Optional, important, input parameter.
//                                              Default is 0. Usually, your font will start with
//                                              a different glyph, so a good idea to provide one
//                                              if you want everything to work as expected.
// cbGlyphAllowedVerticalBlanks
//                   -  Used with:
//                      ConvertBMPFileToBDF  -  Optional input parameter specifying the number 
//                                              of allowed vertical blank lines without
//                                              considering movement to the next glyph. Some 
//                                              fonts are not horizontally contiguous. When 
//                                              using this, ensure your input BMP has space 
//                                              betweem the characters exceeding this value
//                                              or else you will end up short on the number
//                                              of characters than you were expecting.
//                                              Default is 0.
// cbDescentPixels   -  Used with:
//                      ConvertBMPFileToBDF  -  Optional input parameter specifying the number
//                                              of pixels used for a descent value below 
//                                              below the baseline. Default is 0.
// cbGenerateFixedWidth
//                   -  Used with:
//                      ConvertBMPFileToBDF  -  Optional input parameter specifying to make a 
//                                              fixed width font. When used it will cause each 
//                                              bounding box to be the same width, even if the
//                                              character is smaller than the box. Default is 
//                                              a variable width font.
// cbUpperBlankPad   -  Used with:
//                      ConvertBMPFileToBDF  -  Optional input parameter specifying how many
//                                              padding pixels to use above the top pixel 
//                                              of the glyph. Default is 0.
// cbLowerBlankPad   -  Used with:
//                      ConvertBMPFileToBDF  -  Optional input parameter specifying how many
//                                              padding pixels to use below the bottom pixel 
//                                              of the glyph. Default is 0.
// cbLeftBlankPad    -  Used with:
//                      ConvertBMPFileToBDF  -  Optional input parameter specifying how many
//                                              padding pixels to use to the left of the 
//                                              leftmost pixel of the glyph. Default is 0.
// cbRightBlankPad   -  Used with:
//                      ConvertBMPFileToBDF  -  Optional input parameter specifying how many
//                                              padding pixels to use to the right of the 
//                                              rightmost pixel of the glyph. Default is 0.
//
// ***************************************************************************************************************************
// Extra Documentation
// ***************************************************************************************************************************
//
// ---------------------------------------------------------------------------------------------------------------------------
// Definition of Terms
// ---------------------------------------------------------------------------------------------------------------------------
//
// Since it is a bitmapped font map, it should be easier to express terms in an ascii 
// representation of the expected incoming bitmap format for conversion.
// The bitmap cna be a bottom down or top up bitmap. All of the glyphs should remain on a single 
// line meaning only one row for all the glyphs. This is an ascii representation of a 2 glyph
// input bitmap with a guide on the top representing one column per pixel. A # represents a 
// filled pixel, or color of black in the input bitmap. A blank spot represents a blank or
// a non-black pixel on the bitmap. It helps to view this with a fixed width font.
//
// 0123456789012345
//    ##   ######  
//   # #         # 
//     #         # 
//     #         # 
//     #    #####  
//     #   #       
//     #   #       
//  ######  ###### 
//
// That's a two glyph bitmap that would be fed in. Likely the value fed to the starting
// char would be Decimal 49 or the ascii representation of the character '1'. On to the
// definitions of some of the terms from the BMPFontOptions structure.
// Again, much easier to describe with a picture. Let's take the letters A and g or the 
// periodic table description of the element Silver to illustrate the point. A # 
// represents a filled pixel, or color of black in the input bitmap. A blank spot 
// represents a blank or a non-black pixel on the bitmap. It helps to view this with a 
// fixed width font.
//
// cbUpperBlankPad ----|    ------------------ cbGlyphAllowedVerticalBlanks 
// # of leading        |    |                  if 1 or more, treated as 1 glyph
// blank spaces   ---------------------        if 0, treated as 2 glyphs
// above drawing  |    #              |---------------------------------
// pixels         |   ###             |                                 |
// (shown 0)      |   # #       ### # |                                 |
//                |   #  #     #   ## |                                 |
// cbLeftBlankPad |  #   #    #     # | cbRightBlankPad                 |- Ascent 
//  # of leading  |  #    #   #     # | # of trailing blank spaces      | (calculated)
//  blank spaces  | #######   #     # | after drawing pixels. (shown 1) |
//  before        | #      #  #     # |                                 |
// drawing pixels | #      #   #   ## |                                 |
//  (shown 0)     |#        #  #### # |---------------------------------
//                |                 # |--
//                |                #  |  |- cbDescentPixels (3 here)
//              --|            ####   |--
//              | ---------------------
//       |-------  |        | |      |
//       |          --------   ------
// cbLowerBlankPad     |         |
//   # of trailing   width      width2
//   blank spaces    |               |
//   after drawing    ---------------
//      pixels               |
//    (shown 0)      cbGenerateFixedWidth
//                   if 1, Padding would be added to the g character
//                         to make it same width as A
//                   if 0, g would occupy a smaller bounding box than A
//
// ---------------------------------------------------------------------------------------------------------------------------
// Interface Usage Example in C
// ---------------------------------------------------------------------------------------------------------------------------
//
// BMPBDFFileInfo bbfi;
// BMPFontOptions bfo;
// unsigned int nRet, nDetailedError;
//
// memset((void *)&bbfi, 0, sizeof(bbfi);
// memset((void *)&bfo, 0, sizeof(bfo);
//
// bbfi.szBMPFile = "BMPFont.bmp"; /* Or however you get the input file */
// /* Leaving szBDFFile blank in this case, so "BMPFont.bdf" will be used */
// 
// bfo.szFontName = "My Greatest Font Ever";
// bfo.wFirstGlyphValue = 0xac00;
// bfo.cbGlyphAllowedVerticalBlanks = 2; /* 2 blanks allowed in this bitmap */
// bfo.cbDescentPixels = 2; /* 2 descent pixels */
// bfo.cbGenerateFixedWidth = 1; /* These are fixed width */
// bfo.cbUpperBlankPad = 1; /* Pad the top with 1 blank pixel */
// bfo.cbLowerBlankPad = 0; /* Do not pad the bottom */
// bfo.cbLeftBlankPad = 1; /* Pad the left edge */
// bfo.cbRightBlankPad = 0; /* Do not pad the left edge */
// if( (nRet = IBMPTOBDF_ConvertBMPFileToBDF(pBMP2BDF, &bbfi, &bfo, &nDetailedError)) != NA_SUCCESS ){
//    /* Handle Error case, nDetailedError should be filled */
// }else{
//    /* Handle Successful case */
// }
//

#endif // !_IBMPTOBDF_HEADER_
