// File: bmptobdfdll.h
//
// Header File for loading the IBMPToBDF interface from Dll
//
// Copyright (c) 2008
//
#ifndef _BMPTOBDFDLL_HEADER_
#define _BMPTOBDFDLL_HEADER_

// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the BMPTOBDFDLL_EXPORTS
// symbol defined on the command line. this symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// BMPTOBDFDLL_API functions as being imported from a DLL, wheras this DLL sees symbols
// defined with this macro as being exported.
#ifdef BMPTOBDFDLL_EXPORTS
#define BMPTOBDFDLL_API __declspec(dllexport)
#else
#define BMPTOBDFDLL_API __declspec(dllimport)
#endif

typedef int (*pfnCodeUnitLoader)(void *, unsigned int, void **);

#ifdef __cplusplus
extern "C" {
#endif
BMPTOBDFDLL_API int CodeUnitLoader(void * pReserved, unsigned int clsID, void ** ppObj);
#ifdef __cplusplus
}
#endif


// ---------------------------------------------------------------------------------------------------------------------------
// CodeUnitLoader
// ---------------------------------------------------------------------------------------------------------------------------
// 
// int CodeUnitLoader(void * pReserved, unsigned int clsID, void ** ppObj);
//
// Description:
// Creates a new interface for IBMPToBDF with its own private data. Non-static or global.
// 
// Parameters:
// pReserved   -  Must be NULL
// clsID       -  Unique ID for the class. Ex: JCLSID_IBMPTOBDF_V1 for Version 1 of the Interface
// ppObj       -  Pointer to fill with the interface
//
// Return Values:
// NA_SUCCESS        -  Interface created successfully and interface object returned in ppObj
// NAERR_NOTSUPPORT  -  Interface requested not supported. Double check your clsID
// 
// ***************************************************************************************************************************
// Extra Documentation
// ***************************************************************************************************************************
//
// ---------------------------------------------------------------------------------------------------------------------------
// Interface Usage Example in C
// ---------------------------------------------------------------------------------------------------------------------------
//
// IBMPToBDF * pBMP2BDF = NULL;
// HMODULE     hm = NULL;
//
// hm = LoadLibrary(_T("bmptobdf.dll")); // store hm somewhere for eventual release
// if( hm != NULL ){
//    pfnCodeUnitLoader pfnCUL = (pfnCodeUnitLoader)GetProcAddress(hm, _T("CodeUnitLoader"));
//    if( pfnCUL ){
//       if( pfnCUL(NULL, JCLSID_IBMPTOBDF_V1, (void **)&pBMP2BDF) == NA_SUCCESS ){
//          /* Use the Interface */
//          /* For more information on usage asee IBMPToBDF.h */
//          /* When done, release it to prevent memory leaks */
//          IBMPTOBDF_Release( pBMP2BDF );
//       }
//    }
//    /* Eventually free the DLL, might be good to keep globally on hand and manage
//       separately, so no constant load and free of the library */
//    FreeLibrary( hm );
// }
//

#endif //!_BMPTOBDFDLL_HEADER_